﻿#pragma once

#include <SFML/Graphics.hpp>

class IState;

class IStateManager {
public:
	void set_next_state(IState state);
};

class IState {
public:
	IState(IStateManager& state_manager);
	bool do_step();
protected:
	IStateManager m_state_manager;
};

class Application: IStateManager {
public:
	int run();
private:
	void set_next_state(IState* ptr_state);
	void apply_deffer_state_change();
	IState* m_ptr_state_next;
	IState* m_ptr_state_current;
};

class IWindowKeeper {
protected:
	sf::RenderWindow m_window;
	void event_handling();
	void update();
	void render();
public:
	IWindowKeeper(sf::VideoMode video_mode, std::string window_title);
};

class ISelectCommand {
public:
	void execute();
};

class IDrawable {
public:
	void draw_into(sf::RenderWindow& window);
};

class MyFont {
private:
	sf::Font m_font;
	MyFont();
public:
	sf::Font& instance();
};


class Button : IDrawable {
private:
	sf::Text m_text;
	sf::RectangleShape m_shape;
	ISelectCommand* m_ptr_command;
public:
	Button();
	void set(sf::Vector2f pos, std::string text, size_t font_size, ISelectCommand ptr_command);
	void select();
	void unselect();
	void is_position_in(sf::Vector2f pos);
	void push();
};


class Menu {
private:
	Button m_buttons[4];
public:
	Menu(IStateManager& state_manager);
	void process_mouse(sf::Vector2f pos, bool is_pressed);
};

class SelectState : IState, IWindowKeeper {
public:
	SelectState(IStateManager& state_manager, sf::VideoMode video_mode, std::string window_title);
private:
	Menu m_menu;
};

class GameState : IState, IWindowKeeper {
public:

	GameState(IStateManager& state_manager, sf::VideoMode video_mode, std::string window_title);

};

class ExitState : IState {
public:
	ExitState(IStateManager& state_manager);
};

